////////////////////////////////////////////////////////////////////////////
////                             LCD.C                                  ////
////                 Driver for common LCD modules                      ////
////                                                                    ////
////  lcd_init()   Must be called before any other function.            ////
////                                                                    ////
////  lcd_putc(c)  Will display c on the next position of the LCD.      ////
////                     The following have special meaning:            ////
////                      \f  Clear display                             ////
////                      \n  Go to start of second line                ////
////                      \b  Move back one position                    ////
////                                                                    ////
////  lcd_gotoxy(x,y) Set write position on LCD (upper left is 1,1)     ////
////                                                                    ////
////  deleted -> lcd_getc(x,y)   Returns character at position x,y on LCD          ////
////                                                                    ////
////////////////////////////////////////////////////////////////////////////
////        (C) Copyright 1996,1997 Custom Computer Services            ////
//// This source code may only be used by licensed users of the CCS C   ////
//// compiler.  This source code may only be distributed to other       ////
//// licensed users of the CCS C compiler.  No other use, reproduction  ////
//// or distribution is permitted without written permission.           ////
//// Derivative programs created using this software in object code     ////
//// form are not restricted in any way.                                ////
////////////////////////////////////////////////////////////////////////////

// As defined in the following structure the pin connection is as follows:
//     B0  enable
//     B1  rs
//     B2  rw
//	   B3  bus_rw   0= gate external bus
//     B4  D4
//     B5  D5
//     B6  D6
//     B7  D7
//
//   LCD pins D0-D3 are not used and PIC B3 is not used.

struct lcd_pin_map {                 // This structure is overlayed
           BOOLEAN enable;           // on to an I/O port to gain
           BOOLEAN rs;               // access to the LCD pins.
           BOOLEAN rw;      // nu  	 // The bits are allocated from
           BOOLEAN bus_rw;           // low order up.  ENABLE will
           int     data : 4;         // be pin B0.
        } lcd;
									// bus_rw controls drive state of
									// extern bus through 74125
									// 0 gates bus

#byte lcd = 6                        // This puts the entire structure
                                     // on to port B (at address 6)

#define lcd_type 2           // 0=5x7, 1=5x10, 2=2 lines
#define lcd_line_two 0x40    // LCD RAM address for the second line


byte CONST LCD_INIT_STRING[4] = {0x20 | (lcd_type << 2), 0xc, 1, 6};
                             // These bytes need to be sent to the LCD
                             // to start it up.


                             // The following are used for setting
                             // the I/O port direction register.

// changed following to make RB2 always input -- bit for init mode
STRUCT lcd_pin_map const LCD_WRITE = {0,0,1,0,0}; // For write mode all pins are out
STRUCT lcd_pin_map const LCD_READ = {0,0,1,0,15}; // For read mode data pins are in



byte lcd_read_bus() {
      byte low,high;

      set_tris_b(LCD_READ);
      lcd.bus_rw = 0;		// gate bus
      delay_cycles(1);
      low = lcd.data;
      lcd.bus_rw = 1;		// block bus
	  delay_cycles(1);
      set_tris_b(LCD_WRITE);
      return(low);
}


void lcd_send_nibble( byte n ) {
      lcd.enable = 1;
      delay_cycles(1);
      lcd.data = n;
      delay_us(2);
      lcd.enable = 0;
}


void lcd_send_byte( byte address, byte n ) {
	// address: 0=command, 1=data

      lcd.enable = 0;
      lcd.rs = address;
      delay_cycles(1);
      lcd_send_nibble(n >> 4);
      lcd_send_nibble(n & 0xf);
	  delay_us(2);
}


void lcd_init() {
    byte i;

    set_tris_b(0xff);	// all inputs
    lcd.bus_rw = 1;		// block bus
	delay_cycles(2);
    lcd.rs = 0;
    lcd.enable = 0;
    set_tris_b(LCD_WRITE);
    delay_ms(15);
    for(i=1;i<=3;++i) {
       lcd_send_nibble(3);
       delay_ms(5);
    }
    lcd_send_nibble(2);
    for(i=0;i<=3;++i)
       lcd_send_byte(0,LCD_INIT_STRING[i]);
}


void lcd_gotoxy( byte x, byte y) {
   byte address;
   // y = line; 1 = 1st line, 2 = 2nd

   if(y==1)
     address=0;
   else
     address=lcd_line_two;
   address+=x-1;
   lcd_send_byte(0,0x80|address);
}

void lcd_putc( char c) {
   switch (c) {
     case '\f'   : lcd_send_byte(0,1);
                   delay_ms(2);
                                           break;
     case '\n'   : lcd_gotoxy(1,2);        break;
     case '\b'   : lcd_send_byte(0,0x10);  break;
     default     : lcd_send_byte(1,c);     break;
   }
}

#if 0
char lcd_getc( byte x, byte y) {
   char value;

    lcd_gotoxy(x,y);
    lcd.rs=1;
    value = lcd_read_byte();
    lcd.rs=0;
    return(value);
}
#endif


